@@ -44,8 +44,9 @@ class QueryablePropertyDescriptor(property):
         if obj is None:
             return self
 
-        # Always check for cached values first regardless of the self.cached
-        # since values will also be cached through annotations.
+        # Always check for cached values first regardless of the associated
+        # property being configured as cached since values will also be cached
+        # through annotation selections.
         if self.has_cached_value(obj):
             return self.get_cached_value(obj)
         if not self.prop.get_value:
@@ -72,8 +73,8 @@ class QueryablePropertyDescriptor(property):
 
     def get_cached_value(self, obj):
         """
-        Get the cached value for this property from the given object. Requires
-        a cached value to be present.
+        Get the cached value for the associated queryable property from the
+        given object. Requires a cached value to be present.
 
         :param django.db.models.Model obj: The object to get the cached value
                                            from.
@@ -83,7 +84,8 @@ class QueryablePropertyDescriptor(property):
 
     def set_cached_value(self, obj, value):
         """
-        Set the cached value for this property on the given object.
+        Set the cached value for the associated queryable property on the given
+        object.
 
         :param django.db.models.Model obj: The object to set the cached value
                                            for.
@@ -93,7 +95,8 @@ class QueryablePropertyDescriptor(property):
 
     def has_cached_value(self, obj):
         """
-        Check if a value for this property is cached on the given object.
+        Check if a value for the associated queryable property is cached on the
+        given object.
 
         :param django.db.models.Model obj: The object to check for a cached
                                            value.
@@ -104,9 +107,9 @@ class QueryablePropertyDescriptor(property):
 
     def clear_cached_value(self, obj):
         """
-        Clear the cached value for this property on the given object. Does not
-        require a cached value to be present and will do nothing if no value is
-        cached.
+        Clear the cached value for the associated queryable property on the
+        given object. Does not require a cached value to be present and will
+        do nothing if no value is cached.
 
         :param django.db.models.Model obj: The object to clear the cached value
                                            on.
@@ -117,11 +120,11 @@ class QueryablePropertyDescriptor(property):
 @six.python_2_unicode_compatible
 class QueryableProperty(object):
     """
-    Base class for all queryable properties, which are basically simple
-    descriptors with some added methods for queryset interaction.
+    Base class for all queryable property definitions, which provide methods
+    for single object as well as queryset interaction.
     """
 
-    cached = False  #: Determines if the result of the getter is cached, like Django's cached_property.
+    cached = False  #: Determines if the result of the getter is cached, like Python's/Django's ``cached_property``.
     setter_cache_behavior = CLEAR_CACHE  #: Determines what happens if the setter of a cached property is used.
     filter_requires_annotation = False  #: Determines if using the property to filter requires annotating first.
 
@@ -294,8 +297,8 @@ class queryable_property(QueryableProperty):
         (``@getter(cached=True)``).
 
         :param function method: The method to decorate.
-        :param cached: If True, values returned by the decorated getter method
-                       will be cached. A value of None means no change.
+        :param cached: If ``True``, values returned by the decorated getter
+                       method will be cached. A value of None means no change.
         :type cached: bool | None
         :return: A cloned queryable property.
         :rtype: queryable_property
@@ -334,13 +337,13 @@ class queryable_property(QueryableProperty):
         used as a parameter-less decorator (``@filter``) or as a decorator with
         keyword arguments (``@filter(requires_annotation=False)``). May be used
         to define a one-for-all filter function or a filter function that will
-        be called for certain lookups only using the `lookups` argument.
+        be called for certain lookups only using the ``lookups`` argument.
 
         :param method: The method to decorate.
         :type method: function | classmethod | staticmethod
-        :param requires_annotation: True if filtering using this queryable
+        :param requires_annotation: ``True`` if filtering using this queryable
                                     property requires its annotation to be
-                                    applied first; otherwise False. None if
+                                    applied first; otherwise ``False``. None if
                                     this information should not be changed.
         :type requires_annotation: bool | None
         :param lookups: If given, the decorated function or method will be used
@@ -348,19 +351,19 @@ class queryable_property(QueryableProperty):
                         the :class:`LookupFilterMixin` to this property if this
                         is used.
         :type lookups: collections.Iterable[str] | None
-        :param boolean: If True, the decorated function or method is expected
-                        to be a simple boolean filter, which doesn't take the
-                        `lookup` and `value` parameters and should always
-                        return a `Q` object representing the positive (i.e.
-                        `True`) filter case. The decorator will automatically
-                        negate the condition if the filter was called with a
-                        `False` value.
+        :param boolean: If ``True``, the decorated function or method is
+                        expected to be a simple boolean filter, which doesn't
+                        take the ``lookup`` and ``value`` parameters and should
+                        always return a ``Q`` object representing the positive
+                        (i.e. ``True``) filter case. The decorator will
+                        automatically negate the condition if the filter was
+                        called with a ``False`` value.
         :type boolean: bool
-        :param remaining_lookups_via_parent: True if lookup-based filters
+        :param remaining_lookups_via_parent: ``True`` if lookup-based filters
                                              should fall back to the base class
                                              implementation for lookups without
                                              a registered filter function;
-                                             otherwise False. None if this
+                                             otherwise ``False``. None if this
                                              information should not be changed.
         :type remaining_lookups_via_parent: bool
         :return: A cloned queryable property.
