@@ -76,12 +76,102 @@ class QueryPath(tuple):
         return Q(**{six.text_type(self): value})
 
 
+class NodeProcessor(object):
+    """
+    Base class for utilities that work with Django's tree nodes.
+    """
+
+    def __init__(self, func):
+        """
+        Initialize a new node processor.
+
+        :param function func: The function that is applied to nodes and/or
+                              their children. The function must take the node
+                              item as its first argument as well as potentially
+                              more context arguments based on the concrete
+                              implementation, which also defines the expected
+                              return value.
+        """
+        self.func = func
+
+    def iter_leaves(self, node):
+        """
+        Iterate over all leaves of the given node, regardless of the depth of
+        potential sub-nodes.
+
+        :param Node node: The node to get the leaves from.
+        :return: A generator yielding 3-tuples for each leaf consisting of the
+                 (sub-)node the leaf belongs to, the index of the item inside
+                 of that leaf and the leaf item itself.
+        :rtype: collections.Iterable[Node, int, object]
+        """
+        for index, child in enumerate(node.children):
+            if isinstance(child, Node):
+                for result in self.iter_leaves(child):
+                    yield result
+            else:
+                yield node, index, child
+
+
+class NodeChecker(NodeProcessor):
+    """
+    A utility to test tree nodes against a condition specified by the
+    configured function, which therefore must return a boolean value indicating
+    whether or not the condition was met.
+    """
+
+    def check_leaves(self, node, **context):
+        """
+        Check the leaves of the given node object against the configured
+        function.
+
+        :param Node node: The node whose leaves should be checked.
+        :param context: Additional context parameters that will be passed
+                        through to the configured function.
+        :return: True if the condition matches at least one leaf; otherwise
+                 False.
+        :rtype: bool
+        """
+        for branch_node, index, leaf in self.iter_leaves(node):
+            if self.func(leaf, **context):
+                return True
+        return False
+
+
+class NodeModifier(NodeProcessor):
+    """
+    A utility to modify the tree nodes using the configured function, which
+    therefore must return the replacement value for a given item.
+    """
+
+    def modify_leaves(self, node, copy=True, **context):
+        """
+        Modify the leaves of the given node object using the configured
+        function.
+
+        :param Node node: The node whose leaves should be modified.
+        :param bool copy: If True, a copy of the given node will be created and
+                          modified, leaving the original node untouched. If
+                          False, the original node will be modified in place.
+        :param context: Additional context parameters that will be passed
+                        through to the configured function.
+        :return: The modified node.
+        :rtype: Node
+        """
+        if copy:
+            node = deepcopy(node)
+        for branch_node, index, leaf in self.iter_leaves(node):
+            branch_node.children[index] = self.func(leaf, **context)
+        return node
+
+
 class QueryablePropertyReference(namedtuple('QueryablePropertyReference', 'property model relation_path')):
     """
     A reference to a queryable property that also holds the path to reach the
     property across relations.
     """
     __slots__ = ()
+    node_modifier = NodeModifier(lambda item, ref: (six.text_type(ref.relation_path + item[0]), item[1]))
 
     @property
     def full_path(self):
@@ -128,9 +218,7 @@ class QueryablePropertyReference(namedtuple('QueryablePropertyReference', 'prope
             # If the resolved property belongs to a related model, all actual
             # conditions in the returned Q object must be modified to use the
             # current relation path as prefix.
-            def prefix_condition(item):
-                return six.text_type(self.relation_path + item[0]), item[1]
-            q_obj = TreeNodeProcessor(q_obj).modify_leaves(prefix_condition)
+            q_obj = self.node_modifier.modify_leaves(q_obj, ref=self)
         return q_obj
 
     def get_annotation(self):
@@ -217,7 +305,7 @@ class InjectableMixin(object):
         return created_class
 
     @classmethod
-    def inject_into_object(cls, obj, class_name=None):
+    def inject_into_object(cls, obj, class_name=None, init=True):
         """
         Update the given object's class by dynamically generating a new class
         based on the object's original class and this mixin class and changing
@@ -231,12 +319,16 @@ class InjectableMixin(object):
                                name of the dynamically created class will be
                                the one of the object's original class. Will
                                be applied if a new class is created.
+        :param bool init: Whether or not to perform the initialization of
+                          injected attributes if the object's class was
+                          changed.
         :return: The (potentially) modified object.
         """
         new_class = cls.mix_with_class(obj.__class__, class_name)
         if new_class is not obj.__class__:
             obj.__class__ = new_class
-            obj.init_injected_attrs()
+            if init:
+                obj.init_injected_attrs()
         return obj
 
 
@@ -259,68 +351,12 @@ def _unpickle_injected_object(base_class, mixin_class, class_name=None):
              object's state).
     """
     obj = base_class.__new__(base_class, ())
-    return mixin_class.inject_into_object(obj, class_name)
+    return mixin_class.inject_into_object(obj, class_name, init=False)
 
 
 _unpickle_injected_object.__safe_for_unpickling__ = True
 
 
-class TreeNodeProcessor(object):
-    """
-    A utility class to simplify working with Django's Node objects.
-    """
-
-    def __init__(self, node):
-        """
-        Initialize a new node processor for the given node.
-
-        :param Node node: The node to process.
-        """
-        self.node = node
-
-    def check_leaves(self, predicate):
-        """
-        Check if any leaf of this processor's node matches the given predicate.
-
-        :param function predicate: A function that the leaves of the node will
-                                   be tested against. Must take a single
-                                   parameter that represents the leaf value.
-        :return: True if any leaf matched the predicate; otherwise False.
-        :rtype: bool
-        """
-        for child in self.node.children:
-            if isinstance(child, Node) and TreeNodeProcessor(child).check_leaves(predicate):
-                return True
-            elif not isinstance(child, Node) and predicate(child):
-                return True
-        return False
-
-    def modify_leaves(self, modifier, copy=True):
-        """
-        Modify all leaves of this processor's node modifier callable.
-
-        :param function modifier: A callable that will be called for every
-                                  encountered actual node value (not subnodes)
-                                  with that value as its only parameter. It
-                                  must returned the replacement value for the
-                                  given value.
-        :param bool copy: Whether to create a copy of the original node and
-                          modify this copy instead of modifying the original
-                          node in place.
-        :return: The modified node or node copy.
-        :rtype: Node
-        """
-        if copy:
-            return TreeNodeProcessor(deepcopy(self.node)).modify_leaves(modifier, copy=False)
-
-        for index, child in enumerate(self.node.children):
-            if isinstance(child, Node):
-                TreeNodeProcessor(child).modify_leaves(modifier, copy=False)
-            else:
-                self.node.children[index] = modifier(child)
-        return self.node
-
-
 class ModelAttributeGetter(object):
     """
     An attribute getter akin to :func:`operator.attrgetter` specifically
