@@ -8,7 +8,8 @@ from django.db import models
 from queryable_properties.managers import QueryablePropertiesManager
 from queryable_properties.properties import (
     AggregateProperty, AnnotationGetterMixin, AnnotationMixin, AnnotationProperty, LookupFilterMixin, QueryableProperty,
-    queryable_property, RangeCheckProperty, RelatedExistenceCheckProperty, SetterMixin, UpdateMixin, ValueCheckProperty
+    queryable_property, RangeCheckProperty, RelatedExistenceCheckProperty, SetterMixin, SubqueryExistenceCheckProperty,
+    SubqueryFieldProperty, UpdateMixin, ValueCheckProperty
 )
 from ..dummy_lib.models import ReleaseTypeModel
 
@@ -29,20 +30,6 @@ class DummyProperty(SetterMixin, QueryableProperty):
         return -1  # A value < 0 to test the CACHE_RETURN_VALUE setter behavior
 
 
-class HighestVersionProperty(AnnotationMixin, QueryableProperty):
-
-    def get_value(self, obj):
-        try:
-            return obj.versions.order_by('-major', '-minor', '-patch')[0].version
-        except IndexError:
-            return None
-
-    def get_annotation(self, cls):
-        queryset = VersionWithClassBasedProperties.objects.select_properties('version')
-        queryset = queryset.filter(application=models.OuterRef('pk')).order_by('-major', '-minor', '-patch')
-        return models.Subquery(queryset.values('version')[:1], output_field=models.CharField())
-
-
 class VersionCountProperty(AnnotationGetterMixin, QueryableProperty):
 
     def get_annotation(self, cls):
@@ -143,6 +130,9 @@ class CategoryWithClassBasedProperties(Category):
 
     has_versions = RelatedExistenceCheckProperty('applications__versions')
     version_count = AnnotationProperty(models.Count('applications__versions'))
+    has_v2 = SubqueryExistenceCheckProperty(
+        lambda: VersionWithClassBasedProperties.objects.filter(application__categories=models.OuterRef('pk'), major=2)
+    )
     circular = CircularProperty()
 
     class Meta:
@@ -178,7 +168,13 @@ class ApplicationWithClassBasedProperties(Application):
 
     objects = QueryablePropertiesManager()
 
-    highest_version = HighestVersionProperty()
+    highest_version = SubqueryFieldProperty(
+        lambda: (VersionWithClassBasedProperties.objects.select_properties('version')
+                                                        .filter(application=models.OuterRef('pk'))
+                                                        .order_by('-major', '-minor', '-patch')),
+        field_name='version',
+        output_field=models.CharField()
+    )
     version_count = VersionCountProperty()
     major_sum = AggregateProperty(models.Sum('versions__major'))
     major_avg = MajorAverageProperty()
